
#include "LPC214x.h"                        /* LPC21xx definitions */
#include "type.h"
#include "irq.h"
#include "extint.h"
#include "led.h"
/*
	P0.16 EINT0 FIQ DCLOCK
	P0.15 EINT2 	HD
	P0.20 EINT3	VD
	P0.21 PWM5
	P0.10 Reset
*/
volatile DWORD eint1_counter;
unsigned char bild[24577];
unsigned int bild_count;


void __attribute__ ((interrupt("FIQ"),long_call, section (".fastrun")))FIQ_Handler (void) 
{
	EXTINT = EINT0;		/* clear interrupt */
	if(FIOPIN0 & 32768)
	{
		unsigned int data=FIOPIN1;
		bild[bild_count]=(data>>16)&0xFF;
		bild_count++;
		if(bild_count==24577)
			bild_count=0;
	}
	if(!(FIOPIN0 & 1048576))
		bild_count=0;
}

void hd_irq (void) __irq
{
	EXTINT = EINT2;		/* clear interrupt */
	IENABLE;			/* handles nested interrupt */
    
    
	IDISABLE;
	VICVectAddr = 0;		/* Acknowledge Interrupt */
}

void vd_irq (void) __irq
{
	EXTINT = EINT3;		/* clear interrupt */
	IENABLE;			/* handles nested interrupt */
    
	if ( eint1_counter ==1)	
	{
		led_off();
		eint1_counter=0;
		return;
	}
	else
	{
		led_on();
		eint1_counter=1;
		return;
	}
	
	bild_count=0;
	IDISABLE;
	VICVectAddr = 0;		/* Acknowledge Interrupt */
}

void FirqInit(void)
{
	VICIntEnClr = 1 << EINT0_INT;	/* Disable Interrupt */
	PINSEL1 |= 0x00000001;		/* P0.16 Als interrupt */

	EXTMODE |= EINT0_EDGE;		/* INT0 edge or level trigger */
	EXTPOLAR |= EINT0_RISING;	/* INT0 rising or faling  edge  1->rising 0->falling*/
	
	VICIntSelect |= (1<<EINT0_INT);	//FIQ Install bit
	
	VICIntEnable |= (1 << EINT0_INT);	/* Enable Interrupt */
	
}



void Kamera_irq_init(void)
{
	VICIntEnClr = 1 << EINT2_INT;	/* Disable Interrupt */
	VICIntEnClr = 1 << EINT3_INT;	/* Disable Interrupt */

	
	PINSEL0 |= (2<<30);		/* P0.15  EINT2*/
	PINSEL1 |= (3<<8);		/* P0.20  EINT3*/


	EXTMODE |= EINT2_EDGE;		/* INT1 edge trigger */
	EXTMODE |= EINT3_EDGE;		/* INT1 edge trigger */
	EXTPOLAR |= EINT2_RISING;	/* INT1 is falling edge by default */
	EXTPOLAR |= EINT3_RISING;	/* INT1 is falling edge by default */

	if ( install_irq( EINT2_INT, (void *)hd_irq ) == FALSE )//HD
		return ;
	if ( install_irq( EINT3_INT, (void *)vd_irq ) == FALSE )//VD
		return ;

	VICIntEnable |= 1 << EINT2_INT;	/* Enable Interrupt */
	VICIntEnable |= 1 << EINT3_INT;	/* Enable Interrupt */
}



